unit IDEImageConsistencyChecks;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil;

type

  TInconsistency = (icNotPhysicalAvailable, icNotInResourceFileList);

const
  icNotPhysicalAvailableStr  = 'File is in resource file list, but not in image directory';
  icNotInResourceFileListStr = 'File is available in image directory, but not in a resource file list';

  InconsistencyStrings: array [Low(TInconsistency)..High(TInconsistency)] of String = (
    icNotPhysicalAvailableStr,
    icNotInResourceFileListStr);

type

  TInconsistencyEnvent = procedure(const Str: String; Inconsistency: TInconsistency) of object;

  { TIDEImageConsistencyCheck }

  TIDEImageConsistencyCheck = class
  private
    FAllFilesInImagesDirectory: TStringList;
    FAllFilesInFileLists: TStringList;
    FImagesDirectory: String;
    FOnInconsistency: TInconsistencyEnvent;
    function AsResourceNeeded(FileName: String): Boolean;
    procedure CheckInconsistency;
    procedure FindAllFilesInImagesDirectory;
    procedure FindAllFilesInFileLists;
    procedure RemoveUnneedFilesFromFileList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure StartCheck;
    property ImagesDirectory: String read FImagesDirectory write FImagesDirectory;
    property OnInconsistency: TInconsistencyEnvent read FOnInconsistency write FOnInconsistency;
  end;

implementation

{ TIDEImageConsistencyCheck }

function TIDEImageConsistencyCheck.AsResourceNeeded(FileName: String): Boolean;
var
  Extension: String;
begin
  Extension := ExtractFileExt(FileName);
  case Extension of
    '.png',
    '.xpm',
    '.ico': Result := True;
  else
    Result := False;
  end;
end;

procedure TIDEImageConsistencyCheck.CheckInconsistency;
var
  i: Integer;
begin
  for i := 0 to FAllFilesInFileLists.Count - 1 do
    if Pos(ExtractFileName(FAllFilesInFileLists[i]), FAllFilesInImagesDirectory.Text) = 0 then
      FOnInconsistency(FAllFilesInFileLists[i], icNotPhysicalAvailable);

  for i := 0 to FAllFilesInImagesDirectory.Count - 1 do
    if Pos(ExtractFileName(FAllFilesInImagesDirectory[i]), FAllFilesInFileLists.Text) = 0 then
      FOnInconsistency(FAllFilesInImagesDirectory[i], icNotInResourceFileList);
end;

procedure TIDEImageConsistencyCheck.FindAllFilesInImagesDirectory;
begin
  FAllFilesInImagesDirectory := FindAllFiles(FImagesDirectory);
  FAllFilesInImagesDirectory.Text := StringReplace(FAllFilesInImagesDirectory.Text,
                                     FImagesDirectory + PathDelim,
                                     '',
                                     [rfReplaceAll]);
end;

procedure TIDEImageConsistencyCheck.FindAllFilesInFileLists;
var
  i, i2: Integer;
  DummyFileList: TStringList;
begin
  DummyFileList := TStringList.Create;
  try
    FAllFilesInFileLists.Clear;
    for i := 0 to FAllFilesInImagesDirectory.Count - 1 do
      if pos('list.txt', FAllFilesInImagesDirectory[i]) > 0 then
      begin
        DummyFileList.LoadFromFile(FImagesDirectory + PathDelim + FAllFilesInImagesDirectory[i]);
        for i2 := 0 to DummyFileList.Count - 1 do
          FAllFilesInFileLists.Add(DummyFileList[i2]);
      end;
  finally
    DummyFileList.Free;
  end;
end;

procedure TIDEImageConsistencyCheck.RemoveUnneedFilesFromFileList;
var
  i: Integer;
begin
  for i := FAllFilesInImagesDirectory.Count - 1 downto 0 do
    if not AsResourceNeeded(FAllFilesInImagesDirectory[i]) then
      FAllFilesInImagesDirectory.Delete(i);

  for i := FAllFilesInFileLists.Count - 1 downto 0 do
    if FAllFilesInFileLists[i] = '' then
      FAllFilesInFileLists.Delete(i);
end;

constructor TIDEImageConsistencyCheck.Create;
begin
  FAllFilesInImagesDirectory := TStringList.Create;
  FAllFilesInFileLists := TStringList.Create;
  FImagesDirectory := '';
end;

destructor TIDEImageConsistencyCheck.Destroy;
begin
  FAllFilesInFileLists.Free;
  FAllFilesInImagesDirectory.Free;
  inherited Destroy;
end;

procedure TIDEImageConsistencyCheck.StartCheck;
begin
  if FImagesDirectory = '' then Exit;
  if not Assigned(FOnInconsistency) then
    raise Exception.Create('No event handler for OnInconsistency assigned');
  FindAllFilesInImagesDirectory;
  FindAllFilesInFileLists;
  RemoveUnneedFilesFromFileList;
  CheckInconsistency;
end;

end.

